/******************************************************************************
 * ASP DIGITAL (c) 2006
 * All rights reserved.
 *
 * Graphic LCD driver
 * Filename: display.c
 *
 * Display Control.
 *
 * Created 2 Aug 2006
 * revision:
 * 06 Aug 2006: add GLCD_ prefixes.
 *  Move WriteChar here.
 * 08 Aug 2006: add CopyLine();
 *  WriteChar addresses things in terms of characters, not pixels.
 *  Don't return structures! (Keil chokes.)
 * 09 Aug 2006: Umm, the display buffer is 8k bytes, or 65536 bits, deep
 *  (because 480x128 = 61,440 bits).
 ****************************************************************************
 * Display buffer memory format:
 * The display buffer is divided into four buffers, one for each quadrant. 
 * The memories look to the display control logic as four one-bit wide jobs,
 * since the display controller grabs one bit at a time.
 * The memories look to the micro like 8-bit wide memories, so we can write to
 * eight pixels at a time. 
 *
 * Quadrants:
 * +--------------+--------------+
 * |       0      |       1      |
 * +--------------+--------------+
 * |       2      |       3      |
 * +--------------+--------------+
 ****************************************************************************/
#include <C8051F340.h>
#include "display.h"
#include "charmap.h"

volatile unsigned char xdata dispbuf[8192] _at_ 0x1000;

/*
 * ClearMemory():
 *
 */
void 
GLCD_ClearMemory(void)
{
    int i;
    for (i = 0; i < 8192; i++)
        *(dispbuf + i) = 0x00;

} // ClearMemory()

/*
 * CalcAddress():
 * Remember that the control accesses the memory by reading all four bits
 * for each quadrant at once.  It sees the memory for each quadrant as one
 * long string of 15,360 bits.  The remaining 16,384-15,360=1,024 bits are
 * not used.
 * This means that, from the microcontroller's point-of-view, the memory 
 * map wraps in a funny way.  The byte that contains row 0, column 239 is
 * contiguous with the byte that contains row 1, column 0.  Then there's a
 * gap between the byte with row 63, column 239 and row 64, column 0.
 */
unsigned int 
GLCD_CalcAddress(
    pixel_tp Cursor)
{
    unsigned int addr;
    if (Cursor.x > (XMAX / 2))
    {
        addr = (Cursor.x - (XMAX+1)/2)/8;

        if (Cursor.y > (YMAX /2))
        {
            // quadrant = 3, lower right:
            addr += 0x1800 + ((Cursor.y - (YMAX+1)/2) * 30);
        }
        else
        {
            // quadrant = 1, upper right:
            addr += 0x0800 + (Cursor.y * 30);
        }
    }
    else
    {
        addr = (Cursor.x)/8;

        if (Cursor.y > (YMAX /2))
        {
            // quadrant = 2, lower left:
            addr += 0x1000 + ((Cursor.y - (YMAX+1)/2) * 30);
        }
        else
        {
            // quadrant = 0, upper left:
            addr += (Cursor.y * 30);
        }
    }
    return addr;
} // CalcAddress()

/*
 * WritePixel():
 * Write the Pixel to the Cursor location.
 */
void 
GLCD_WritePixel(
    pixel_tp Cursor, 
    bit Pixel)
{
    unsigned int addr;      // byte address of the pixel to change
    unsigned char mask;     // mask to set or clear a pixel

    addr = GLCD_CalcAddress(Cursor);

    // create a mask for the pixel we wish to set or clear:
    mask = (1 << (Cursor.x % 8));

    if (Pixel)
        dispbuf[addr] |= mask;
    else
        dispbuf[addr] &= ~mask;  
} // WritePixel()

/*
 * WriteByte():
 * write eight bits, given as a mask, starting at Cursor.
 * Cursor.x should be on an 8-bit boundary.
 */
void 
GLCD_WriteByte(
    pixel_tp Cursor, 
    unsigned char ByteData)
{
    unsigned int addr;      // byte address of the pixel to change

    addr = GLCD_CalcAddress(Cursor);
    dispbuf[addr] = ByteData;
} // GLCD_WriteByte()

unsigned char
GLCD_ReadByte(
    pixel_tp Cursor)
{
    unsigned int addr;      // byte address of the pixel to change
    unsigned char tmp;

    addr = GLCD_CalcAddress(Cursor);
    tmp = dispbuf[addr];
    return tmp;
} // GLCD_ReadByte

/*
 * convert from a character location to a pixel location.
 */
void 
GLCD_CharLocToPixLoc(
    charloc_tp *ct, 
    pixel_tp *pt)
{
    pt->x = ct->x * 8;
    pt->y = ct->y * 8;
} // GLCD_CharLocToPixLoc

/*
 * Write a character to the screen at the desired location.
 */
void 
GLCD_WriteChar(
    charloc_tp CharLoc, 
    unsigned char CharSel)
{
    unsigned char line;
    pixel_tp PixLoc;

    GLCD_CharLocToPixLoc(&CharLoc, &PixLoc);

    for (line = 0; line < 8; line++)
    {
        GLCD_WriteByte(PixLoc, CHARMAP[CharSel][line]);
        ++PixLoc.y;
    }
} // GLCD_WriteChar()

void 
GLCD_LineCharToCharLoc(
    unsigned char linesel, 
    unsigned char charsel,
    charloc_tp *ct)
{
    ct->x = charsel;
    ct->y = linesel;
} // GLCD_LineChartoCharLoc

void 
GLCD_LineCharToPixel(
    unsigned char linesel, 
    unsigned char charsel, 
    pixel_tp *pt)
{
    pt->x = charsel * 8;
    pt->y = linesel * 8;
} // GLCD_LineCharToPixel()

/*
 * shift all of the lines in the display up one line, and clear the
 * bottom line.
 */
void GLCD_ShiftLinesUp(unsigned char reps)
{
    unsigned char row;          // row counter
    unsigned char col;          // column counter
    unsigned int index;         // computed array index
    unsigned char rep;          // rep counter.
    unsigned char tmpchar;      // temporary storage for the copy

    for (rep = 0; rep < reps; rep++)
    {
        for (row = 0; row < 63; row++)
        {
            for (col = 0; col < 30; col++)
            {
                // quadrant 0, upper left:
                index = ((unsigned int) (row+1) * 30) + col; // src
                tmpchar = dispbuf[index];
                index = ((unsigned int) (row) * 30) + col; // dest
                dispbuf[index] = tmpchar;
                // quadrant 1, upper right:
                index = 2048 + ((unsigned int) (row+1) * 30) + col; // src
                tmpchar = dispbuf[index];
                index = 2048 + ((unsigned int) (row) * 30) + col; // dest
                dispbuf[index] = tmpchar;
            }
        }
    
        // special case: copy row 64 to row 63, which crosses address boundary:
        for (col = 0; col < 30; col++)
        {
            // last row, quadrant 0:
            index = 4096 + col; // row 64 is in q2, which starts at offset 4096
            tmpchar = dispbuf[index];
            index = (63 * 30) + col;  // row number * cols per row + col offset
            dispbuf[index] = tmpchar;
            // last row, quadrant 1:
            index = 6144 + col; // row 64 is in q4, which starts at offset 6144
            tmpchar = dispbuf[index];
            index = 2048 + (63 * 30) + col;  // row number * cols per row + col offset
            dispbuf[index] = tmpchar;
        }
    
        for (row = 0; row < 63; row++) // really rows 65 to 126
        {
            for (col = 0; col < 30; col++)
            {
                // column 2, lower left:
                index = 4096 + ((unsigned int) (row+1) * 30) + col;
                tmpchar = dispbuf[index];
                index = 4096 + ((unsigned int) (row) * 30) + col;
                dispbuf[index] = tmpchar;
                // column 4, lower right:
                index = 6144 + ((unsigned int) (row+1) * 30) + col;
                tmpchar = dispbuf[index];
                index = 6144 + ((unsigned int) (row) * 30) + col;
                dispbuf[index] = tmpchar;
            }
        }   
    
        // special case: last row, fill with zero:
        for (col = 0; col < 30; col++)
        {
            index = 4096 + (63 * 30) + col;
            dispbuf[index] = 0x00;
            index = 6144 + (63 * 30) + col;
            dispbuf[index] = 0x00;
        }
    }
} // GLCD_ShiftLinesUp()

