/******************************************************************************
 * ASP DIGITAL (c) 2006
 * All rights reserved.
 *
 * Graphic LCD driver
 * Filename: main.c
 *
 * Main entry point and loop.
 *
 * Created 29 Jul 2006
 * revision:
 * 07 Aug 2006: Simple terminal display application is working.
 *  App handles RS232 inputs and displays characters.  Backspace and carriage
 *  return are handled as well.
 *
 ****************************************************************************
 * This is the main program entry point for a microcontroller program that
 * makes a GLCD look like a text terminal window. 
 * We receive ASCII characters over the RS-232 UART and display them on the 
 * terminal.  If a PS/2 keyboard is connected to the controller, then we
 * accept characters coming in from the SPI port, too. 
 *
 * There are several elements to displaying text on our 480x128-pixel display.
 *
 * First, we have character generator "ROMs," which are 5x8 maps of the 
 * characters we're able to display.
 * 
 * Next, we need to maintain a current cursor position at all times.  We show
 * the cursor as a solid block.
 *
 * We communicate with the GLCD controller FPGA using the EMIF.  The display
 * buffer is memory-mapped, starting at XDATA location 0x1000 up to 0x1E00.
 * 
 * The buffer memory is interesting, in that the GLCD has four independent 
 * quadrants.  Every vertical sync, the controller initializes its address 
 * counter, and we increment that counter every pixel clock.  Each address
 * read gives us four pixels, one for each quadrant.  However, the micro 
 * accesses the display buffer eight bits at a time, so there's some mapping
 * that has to occur.
 ****************************************************************************
 * C8051F340 Peripherals and Port Configuration:
 * ---------------------------------------------
 * P0.0: SCLK, open-drain, SPI clock input
 * P0.1: MISO, push-pull output, SPI data out to master
 * P0.2: MOSI: open-drain, SPI data in from master
 * P0.3: NSS:  SPI slave select input, active low
 * P0.4: TX0: UART transmit out
 * P0.5: RX0: UART receive in
 * P0.6: SYSCLK: 48 MHz oscillator output
 * P1.6: RD\: EMIF read strobe output
 * P1.7: WR\: EMIF write strobe output
 * P2: EMIF upper address byte (A15..A8)
 * P3: EMIF lower address byte (A7 ..A0)
 * P4: EMIF data I/O byte
 *
 * OSCILLATOR: Internal, H-F oscillator enabled (12 MHz).
 *	Clock multiplier enabled, SYSCLK taken from clock multiplier output.
 * UART0: enabled, baud rate = 9600
 * SPI: slave, interrupt (6) not enabled.
 * Timer 1: enabled, interrupt enabled, paces UART transmit.
 * Timer 3: Interrupt enabled, 16-bit auto-reload, overflow every 25 ms.
 ****************************************************************************
 * Display buffer memory format:
 * The display buffer is divided into four buffers, one for each quadrant. 
 * The memories look to the display control logic as four one-bit wide jobs,
 * since the display controller grabs one bit at a time.
 * The memories look to the micro like 8-bit wide memories, so we can write to
 * eight pixels at a time.
 ****************************************************************************/
#include <C8051F340.h>
#include "init.h"
#include "display.h"
#include ".\uart\uart.h"

extern bit Timer3;

void main(void)
{
    charloc_tp Cursor;
    unsigned char dispchar;
    
	// Disable watchdog:
	PCA0MD = 0x00;
	Timer3 = 0;
	Init_Device();
    GLCD_ClearMemory();
    UartInitFifos();
    Cursor.x = CHARXMIN;
    Cursor.y = CHARYMIN;

    UartWriteTxFifo("hello, world!\n\r", sizeof("hello, world!\n\r")-1);

	while (1)
	{
		if (Timer3)
		{
    		Timer3 = 0;
            if (isUartRxFifoEmpty() != 1)
            {
                dispchar = UartReadRxFifo();
                /*
                 * Handle special characters.
                 */
                switch (dispchar)
                {
                    case 8: // backspace.
                        // first, erase the current cursor:
                        GLCD_WriteChar(Cursor, 32);
                        // If not at tne start of the line, move the cursor to
                        // the previous location:
                        if (Cursor.x != CHARXMIN)
                        {
                            --Cursor.x;
                        }
                        else
                        {
                            // we're at the beginning of a line.  If not the
                            // first line, bump up to the end of the previous
                            // line.
                            if (Cursor.y != CHARYMIN)
                            {
                                --Cursor.y;
                                Cursor.x = CHARXMAX-1;
                            }
                        }
                        GLCD_WriteChar(Cursor, 31); // draw cursor.
                        break;

                    case 13 : // carriage return:
                        // first, erase the current cursor:
                        GLCD_WriteChar(Cursor, 32);
                        // move cursor to the beginning of the line:
                        Cursor.x = CHARXMIN;

                        // See if we're on the last line.
                        if (Cursor.y == CHARYMAX)
                        {
                            // We're on the last line.  We need to shift all
                            // of the lines up.  Do this by reading line 1,
                            // and copy that data to line 0, and then continue
                            // through the last line.
                            GLCD_ShiftLinesUp(8);
                            // and leave the cursor at the start of the last line:
                            Cursor.y = CHARYMAX;
                        }
                        else
                            // since we're not already on the last line, bump 
                            // the cursor to the next line:
                            ++Cursor.y;

                        // finally, draw the cursor:
                        GLCD_WriteChar(Cursor, 31);
                        break;

                    default: // printable characters:
                        GLCD_WriteChar(Cursor, dispchar);
                        // update cursor position:
                        if (Cursor.x == CHARXMAX)
                        {
                            Cursor.x = CHARXMIN;
                            if (Cursor.y == CHARYMAX)
                                Cursor.y = CHARYMIN;
                            else
                                ++Cursor.y;
                        }
                        else
                            ++Cursor.x;
                        GLCD_WriteChar(Cursor, 31); // draw cursor.
                        break;
                } // switch (dispchar)

            } // if UartFifo ...
    	} // if timer3
	} // while()
} // main()
