/*
 * 8051 UART support functions.
 * Created:
 * Modified: 
 *	31 Jan 2006: no unsigned.
 *  30 Mar 2006: fix comments in UartSendNextByte (we use T1 to pace transmit)
 *		Disable Timer 1 interrupt when in UartSendNextByte() so we don't step
 *		on our toes.
 *  10 Jul 2006: back to unsigned.  According to the C51 docs, unsigned is 
 * 		better.
 *		Specify register bank for ISR.
 * 
 * This collection of functions is intended to simplify 8051 UART interfaces.
 * Several functions are included.  They are:
 *
 * UartISR(): this interrupt-service routine is called if the UART interrupt
 * is enabled.  If a receive-complete interrupt occurred, the received byte is
 * written to the receive FIFO.  If a transmit-done interrupt occurred, do 
 * nothing.
 *
 * UartGetChar(): this function returns the character in the serial port 
 * buffer.  It waits until a character is available
 */

#include "uart.h"
#include <C8051F000.H>

unsigned char xdata RxFifo[FDEPTH];
unsigned char RxWrPtr;
unsigned char RxRdPtr;
unsigned char xdata TxFifo[FDEPTH];
unsigned char TxWrPtr;
unsigned char TxRdPtr;
bit TxReady;

void UartInitFifos(void)
{
	unsigned char i;
	RxWrPtr = 0;
	RxRdPtr = 0;
	TxWrPtr = 0;
	TxRdPtr = 0;
	TxReady = 1;
	// clear FIFO contents; good for debug.
	for (i = 0; i < FDEPTH; i++)
	{
		RxFifo[i] = 0x00;
		TxFifo[i] = 0x00;
	}
} // UartInitFifos()

/*
 * Uart ISR. 
 * If a receive interrupt, write the incoming byte to the receive FIFO, and
 * clear the receive interrupt flag.
 * If a transmit interrupt, note it by setting the Transmit Ready flag.
 */
void UartISR(void) interrupt 4 using 3
{
	if (RI)
	{
		RI = 0;
		RxFifo[RxWrPtr] = SBUF;
		RxWrPtr = (RxWrPtr + 1) % FDEPTH;
	}

	if (TI)
	{
		TI = 0;
		TxReady = 1;
	}
} // UartISR()

/*
 * Return true when te 
 */
bit isUartRxFifoEmpty(void)
{
	return (RxWrPtr == RxRdPtr);
} // isUartRxFifoEmpty()

/*
 * Return true when the transmit FIFO is full.
 */
bit isUartTxFifoFull(void)
{
	if (TxWrPtr > TxRdPtr)
		return (TxWrPtr == (TxRdPtr + FDEPTH - 1));
	else
		return (TxWrPtr == (TxRdPtr - 1));
} // isUartTxFifoFull()

/*
 * Return true when the transmit FIFO is empty.
 */
bit isUartTxFifoEmpty(void)
{
	return (TxWrPtr == TxRdPtr);
} // isUartTxFifoEmpty()

/*
 * Read the character at the top of the receive FIFO.
 */
unsigned char UartReadRxFifo(void)
{
	unsigned char retval = RxFifo[RxRdPtr];
	RxRdPtr = (RxRdPtr + 1) % FDEPTH;
	return (retval);
} // UartReadRxFifo()

/*
 * Write cnt chars in str to the transmit FIFO.
 * If the FIFO is full, or fills, wait for it to have space left before
 * writing.
 */
void UartWriteTxFifo(unsigned char *str, unsigned char cnt)
{
	while (cnt != 0)
	{
		if (isUartTxFifoFull() == 0)
		{
			TxFifo[TxWrPtr] = *str;
			str++;
			TxWrPtr = (TxWrPtr + 1) % FDEPTH;
			--cnt;
		}
	} // while (cnt != 0)
} // UartWriteTxFifo()

/*
 * Use Timer 1 as our mechanism to determine when to transmit.
 * When Timer 1 times out, check to see if we have something to send and if
 * the transmitter is ready.  If both cases are true, pop the next value from
 * the transmit FIFO and write it to the serial port.
 * Disable Timer 1's interrupt so we don't step on our toes here.
 */
void UartSendNextByte(void)
{
	ET1 = 0;
	if (TxReady	&& !isUartTxFifoEmpty())
	{
		SBUF = TxFifo[TxRdPtr];
		TxRdPtr = (TxRdPtr + 1) % FDEPTH;
		TxReady = 0;
	}
	ET1 = 1;
} // UartSendNextByte()
